using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Exchange_Component_Highlighting(WindowKey wk, Canvas canvas, ref View view, Model model, SegmentKey modelKey)
        {
            try
            {
            /* Sample: Exchange_Component_Highlighting
	        * This sample will show you how to:
	        * - Walk the component structure of a CAD Model
	        * - Highlight a specific instance of a component
	        * - Highlight all instances of a component
	        * - Build a KeyPath
	        * - Define a Named Style used for highlighting
	        * 
	        * Read more about the topics covered in this sample in our Programming Guide 
	        * - Integrating HPS with HOOPS Exchange
	        * - Section 6.2:		Highlighting
	        * - Section 1.3.1:		Using Keys and Objects
	        * */

            /* This next block of code shows you how to load an Exchange file
            * 1. Create an ImportOptionsKit
            *		NOTE: It is required to import BRep data to access geometry referenced by PMI
            * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
            * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
            * 4. Use a try-catch block to capture any Exchange IO exceptions 
            * 5. Obtain a CADModel object through the Exchange::ImportNotifier */
            HPS.CADModel cadModel = new HPS.CADModel();
            HPS.Exchange.ImportNotifier exchangeNotifier;
            try
            {
                string file_in = INPUT_FOLDER + "/LandingGear.prc";
                HPS.Exchange.ImportOptionsKit importOptionsKit = new HPS.Exchange.ImportOptionsKit();
                importOptionsKit.SetBRepData(true);
                exchangeNotifier = HPS.Exchange.File.Import(file_in, importOptionsKit);
                exchangeNotifier.Wait();

                cadModel = exchangeNotifier.GetCADModel();
            }

            catch (HPS.IOException)
            {
                string error_message;
                error_message = "Exchange Import Unsuccessful";
                modelKey.InsertText(new HPS.Point(0, 0, 0), error_message);
                view.FitWorld();
                view.Update();
            }

            /* This block of code sets up a named style which will be used for highlighting
            * 1. Obtain a portfolio key. The portfolio used will need to be visible from the segment
            *		using the name style defined in it.
            * 2. Create a segment which will contain the attributes for highlighting 
            * 3. Use the DefineNamedStyle method to create a named style in a Portfolio
            *		To create a named style, provide it with a name, and specify the segment key
            *		of the segment containing the attributes used for highlighting */
            HPS.PortfolioKey pKey;
            wk.GetPortfolioControl().ShowTop(out pKey);

            HPS.SegmentKey redSegment = Database.CreateRootSegment();
            redSegment.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1, 0, 0, 1))
                .SetTextColor(new HPS.RGBAColor(1, 0.65f, 0, 1))
                .SetLineColor(new HPS.RGBAColor(1, 0.65f, 0, 1));
            pKey.DefineNamedStyle("redHighlight", redSegment);

            /* This block of code sets up our scene:
             * First we clean-up our scene by deleting the View and Model currently attached to the Layout,
             * Then we setup the Layout to contain two views as follows:
             * 1. Activate the default capture from the CAD Model
             * 2. Add two views to the layout */
            Layout layout = canvas.GetAttachedLayout();
            layout.GetAttachedView().Delete();
            model.Delete();

            View viewLeft = cadModel.ActivateDefaultCapture();
            View viewRight = cadModel.ActivateDefaultCapture();
            layout.AttachViewFront(viewLeft, new HPS.Rectangle(-1, 0, -1, 1));
            layout.AttachViewFront(viewRight, new HPS.Rectangle(0, 1, -1, 1));

            /* This code shows how to walk the component structure of the CAD Model
             * For this particular model we assume that we know where the data we wish to highlight is located
             * The model loaded in this file represents the landing gear of a plane.
             * The landing gear contains four wheels. 
             * Only one wheel prototype exists in the model, and four instances of it are then included
             * where appropriate to represent four wheels without replicating data.
             * 
             * A key path to the specific part will be needed, and can be obtained using the GetKeyPath method */
            HPS.Component[] rootComponent = cadModel.GetSubcomponents();
            HPS.Component[] parts = rootComponent[0].GetSubcomponents();
            HPS.Component[] wheels = parts[2].GetSubcomponents();
            HPS.Component[] wheelInstance = wheels[1].GetSubcomponents();

            /* This code shows how to highlight only one instance of a component
	         * To highlight only one component we build a component path to the specific
	         * component we want to highlight. */

            HighlightOptionsKit highlightOptions = new HighlightOptionsKit();
            highlightOptions.SetStyleName("redHighlight");

            List<Component> components = new List<Component>();
            components.Add(wheelInstance[0]);
            components.Add(wheels[1]);
            components.Add(parts[2]);
            components.Add(rootComponent[0]);
            components.Add(cadModel);

            ComponentPath componentPath = new ComponentPath(components.ToArray());
            componentPath.Highlight(canvas, 0, highlightOptions);

            /* This next block of code performs highlighting 
	         * The KeyPathArray object obtained above will contain eight KeyPath objects:
	         * each one describing the path to one of the four instances of a wheel, 
	         * times two, since the model is included in two views
	         * 
	         * To highlight only the instances present in the view to the right, 
	         * we only highlight the key paths which include the correct view include link*/
            KeyPath[] keyPaths = Component.GetKeyPath(wheelInstance[0]);
            foreach (var keyPath in keyPaths)
            {
                Key[] keys;
                keyPath.ShowKeys(out keys);

                if (keys.Contains(layout.GetAttachedViewIncludeLink(1)))
                    wk.GetHighlightControl().Highlight(keyPath, highlightOptions);
            }

            /* The next block adds text description to both views. For each view the following
	         * operations are performed:
	         * 1. A subsegment is created, and depth range is set on it to make sure that the geometry in this
	         *		segment will always appear on top
	         * 2. Text visibility is enabled through the VisibilityControl
	         * 3. A stretched camera is added to the subsegment
	         * 4. The text attributes are modified as needed using the TextAttributeControl
	         * 5. Text is inserted */
            SegmentKey view1Description = viewLeft.GetSegmentKey().Subsegment();
            view1Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
            view1Description.GetVisibilityControl().SetText(true);
            view1Description.SetCamera(CameraKit.GetDefault());
            view1Description.GetTextAttributeControl().SetAlignment(Text.Alignment.Center);
            view1Description.InsertText(new Point(0, 0.85f, 0), "All instances of the component\n are highlighted in red");


            SegmentKey view2Description = viewRight.GetSegmentKey().Subsegment();
            view2Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
            view2Description.GetVisibilityControl().SetEverything(false).SetText(true);
            view2Description.SetCamera(CameraKit.GetDefault());
            view2Description.GetTextAttributeControl().SetAlignment(Text.Alignment.Center);
            view2Description.InsertText(new Point(0, 0.85f, 0), "Component instance is highlighted in red");

            viewLeft.FitWorld();
            viewRight.FitWorld();
            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}
